@@ -35,7 +35,7 @@ C   Initialize estimates in multivariate local constant aws
 C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine iawsmul(n,y,nn,distm,ih,hinit,bi,ai,kernl)
-C    
+C
 C     n          number of design points
 C     y          observed values at design points
 C     nn         indices of ihinit nearest neighbors
@@ -44,7 +44,7 @@ C     ih         initial number of nearest neighbors
 C     hinit      initial bandwidth (scaled as number of nearest neighbors)
 C     bi         \sum \Psi^T Wi \Psi  (output)
 C     ai         \sum \Psi^T Wi Y     (output)
-C     kernl      discretized localization kernel 
+C     kernl      discretized localization kernel
 C
       integer n,i,j,ja,iz,ih,nn(ih,n)
       real*8 bi(n),ai(n),ha2,az,z,epij,y(n),hinit,ha,kernl(102),
@@ -146,7 +146,7 @@ C     kerns      discretized stochastic  (exponential)
 C     sym      asymmetric or symmetric test (logical)
 C     
       implicit logical (a-z)
-      integer n,i,j,iz,ja,o,nwij,ij,ihakt,nn(ihakt,n)      
+      integer n,i,j,iz,ja,ihakt,nn(ihakt,n)      
       logical sym
       real*8 y(n),theta(n),kerns(102),bi(n),bin(n),ai(n),ain(n),lam,
      1     kernl(102),distm(ihakt,n),epij,lambda,h,wij,z,az,ha2,bii,
@@ -213,7 +213,7 @@ C     kerns      discretized stochastic  (exponential)
 C     sym        asymmetric or symmetric test (logical)
 C     
       implicit logical (a-z)
-      integer n,i,j,iz,ja,o,nwij,ij,ihakt,nn(ihakt,n)      
+      integer n,i,j,iz,ja,ij,ihakt,nn(ihakt,n)      
       logical sym
       real*8 y(n),theta(n),kerns(102),bi(n),bin(n),ai(n),ain(n),lam,
      1    kernl(102),epij,lambda,h,ha,wij,z,az,ha2,bii,thetai
@@ -287,7 +287,7 @@ C
       real*8 y(n),hakt,theta(n),bi(n),ai(n),kernl(102),kerns(102),
      1      lamakt,lam,ltheta(n),lctheta(n)
       integer i,j,ja,je,ih,iz
-      real*8 z,z1,wj,az,swj,swjy,bii,thetai,lthetai,lcthetai,cthetai
+      real*8 z,wj,az,swj,swjy,bii,thetai,lthetai,lcthetai,cthetai
       ih=hakt
       lam=lamakt*1d-2
       if(sym) lam=2*lam
@@ -330,7 +330,7 @@ C
 C   Perform all iterations in local constant univariate aws for bernoulli variables 
 C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
-      subroutine gberuni(y,n,hinit,hincr,hmax,lamakt,eta,theta,ltheta, 
+      subroutine gberuni(y,n,hinit,hincr,hmax,lamakt,eta,theta,ltheta,
      1                   lctheta,bi,ai,kernl,kerns,biold,aiold,sym)
 C   
 C   y        observed values of regression function
@@ -385,27 +385,28 @@ C   Perform one iteration in local constant bivariate aws for bernoulli variable
 C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine lberbi(y,n1,n2,hakt,lamakt,theta,ltheta,lctheta,
-     1                  bi,ai,kernl,kerns,sym)
-C   
+     1                  bi,ai,kernl,kerns,sym,wght)
+C
 C   y        observed values of regression function
 C   n1       number of grid-points in first dimension
 C   n2       number of grid-points in second dimension
 C   hakt     actual bandwidth
-C   lamakt   lambda*sigma2 
+C   lamakt   lambda*sigma2
 C   theta    estimates            (input)
 C   ltheta   log of estimates    (input)
 C   lctheta  log of 1-estimates    (input)
 C   bi       \sum \Psi^T Wi \Psi  (input/output)
 C   ai       \sum \Psi^T Wi Y     (output)
-C   kernl    discretized localization kernel 
-C   kerns    discretized stochastic kernel 
+C   kernl    discretized localization kernel
+C   kerns    discretized stochastic kernel
 C   sym      asymmetric or symmetric test (logical)
 C
       implicit logical (a-z)
       integer n1,n2
       logical sym
       real*8 y(n1,n2),hakt,theta(n1,n2),bi(n1,n2),ai(n1,n2),lam,
-     1      kernl(102),kerns(102),lamakt,ltheta(n1,n2),lctheta(n1,n2)
+     1      kernl(102),kerns(102),lamakt,ltheta(n1,n2),lctheta(n1,n2),
+     2      wght
       integer i1,i2,j1,j2,ja1,ja2,je1,je2,ih1,ih2,iz
       real*8 z,z1,z2,wj,az,swj,swjy,bii,thetai,hakt2,lthetai,
      1       lcthetai,cthetai
@@ -427,7 +428,7 @@ C
             do 2 j1=ja1,je1
                z1=(i1-j1)
                z1=z1*z1
-               ih2=dsqrt(hakt2-z1)
+               ih2=dsqrt(hakt2-z1)/wght
                ja2=max0(1,i2-ih2)
                je2=min0(n2,i2+ih2)
                do 2 j2=ja2,je2
@@ -441,7 +442,7 @@ C  first stochastic term
                   iz=z
                   az=z-iz
                   wj=kerns(iz+1)*(1-az)+kerns(iz+2)*az
-                  z2=(i2-j2)
+                  z2=(i2-j2)*wght
                   z=1.d2*(z1+z2*z2)/hakt2
                   if(z.ge.1.d2) goto 2
                   iz=z
@@ -461,23 +462,24 @@ C   Perform all iterations in local constant bivariate aws for bernoulli variabl
 C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine gberbi(y,n1,n2,hinit,hincr,hmax,lamakt,eta,theta,
-     1              ltheta,lctheta,bi,ai,kernl,kerns,biold,aiold,sym)
-C   
+     1              ltheta,lctheta,bi,ai,kernl,kerns,biold,aiold,sym,
+     2              wght)
+C
 C   y        observed values of regression function
 C   n1       number of grid-points in first dimension
 C   n2       number of grid-points in second dimension
 C   hinit    initial bandwidth
 C   hincr    factor used to increase bandwiths
 C   hmax     maximal bandwidth
-C   lamakt   lambda*sigma2 
+C   lamakt   lambda*sigma2
 C   eta      memory parameter
 C   theta    estimates            (input)
 C   ltheta   log of estimates    (input)
 C   lctheta  log of 1-estimates    (input)
 C   bi       \sum \Psi^T Wi \Psi  (input/output)
 C   ai       \sum \Psi^T Wi Y     (output)
-C   kernl    discretized localization kernel 
-C   kerns    discretized stochastic kernel 
+C   kernl    discretized localization kernel
+C   kerns    discretized stochastic kernel
 C   biold    working array to store old values of bi
 C   aiold    working array to store old values of ai
 C   sym      asymmetric or symmetric test (logical)
@@ -487,7 +489,7 @@ C
       logical sym
       real*8 y(n1,n2),hinit,hincr,hmax,theta(n1,n2),bi(n1,n2),
      1       ai(n1,n2),kernl(102),kerns(102),lamakt,eta,biold(n1,n2),
-     2       aiold(n1,n2),ltheta(n1,n2),lctheta(n1,n2),eps
+     2       aiold(n1,n2),ltheta(n1,n2),lctheta(n1,n2),eps,wght
       integer i1,i2
       real*8 hakt,onemeta
       hakt=hinit*hincr
@@ -495,11 +497,11 @@ C
       eps=1.d-50
 1     do 21 i1=1,n1
          do 21 i2=1,n2
-            ltheta(i1,i2)=dlog(theta(i1,i2)+eps)     
-            lctheta(i1,i2)=dlog(1.d0-theta(i1,i2)+eps)     
+            ltheta(i1,i2)=dlog(theta(i1,i2)+eps)
+            lctheta(i1,i2)=dlog(1.d0-theta(i1,i2)+eps)
 21    continue
       call lberbi(y,n1,n2,hakt,lamakt,theta,ltheta,lctheta,bi,ai,
-     1            kernl,kerns,sym)
+     1            kernl,kerns,sym,wght)
       do 31 i1=1,n1
          do 31 i2=1,n2
             ai(i1,i2)=onemeta*ai(i1,i2)+eta*aiold(i1,i2)
@@ -518,21 +520,21 @@ C   Perform one iteration in local constant trivariate aws for bernoulli variabl
 C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine lbertri(y,n1,n2,n3,hakt,lamakt,theta,ltheta,lctheta,
-     1                  bi,ai,kernl,kerns,sym)
-C   
+     1                  bi,ai,kernl,kerns,sym,wght)
+C
 C   y        observed values of regression function
 C   n1       number of grid-points in first dimension
 C   n2       number of grid-points in second dimension
 C   n3       number of grid-points in third dimension
 C   hakt     actual bandwidth
-C   lamakt   lambda*sigma2 
+C   lamakt   lambda*sigma2
 C   theta    estimates            (input)
 C   ltheta   log of estimates    (input)
 C   lctheta  log of 1-estimates    (input)
 C   bi       \sum \Psi^T Wi \Psi  (input/output)
 C   ai       \sum \Psi^T Wi Y     (output)
-C   kernl    discretized localization kernel 
-C   kerns    discretized stochastic kernel 
+C   kernl    discretized localization kernel
+C   kerns    discretized stochastic kernel
 C   sym      asymmetric or symmetric test (logical)
 C
       implicit logical (a-z)
@@ -540,12 +542,14 @@ C
       logical sym
       real*8 y(n1,n2,n3),hakt,theta(n1,n2,n3),bi(n1,n2,n3),kernl(102),
      1      ai(n1,n2,n3),kerns(102),lamakt,lam,ltheta(n1,n2,n3),
-     2      lctheta(n1,n2,n3)
+     2      lctheta(n1,n2,n3),wght(2)
       integer i1,i2,i3,j1,j2,j3,ja1,ja2,ja3,je1,je2,je3,ih1,ih2,ih3,iz
       real*8 z,z1,z2,z3,wj,az,swj,swjy,bii,thetai,hakt2,lthetai,
-     1       lcthetai,cthetai
+     1       lcthetai,cthetai,wght2,wght3
       ih1=hakt
       hakt2=hakt*hakt
+      wght2=wght(1)
+      wght3=wght(2)
       lam=lamakt*1d-2
       if(sym) lam=2*lam
       do 1 i1=1,n1
@@ -563,13 +567,13 @@ C
                do 2 j1=ja1,je1
                   z1=(i1-j1)
                   z1=z1*z1
-                  ih2=dsqrt(hakt2-z1)
+                  ih2=dsqrt(hakt2-z1)/wght2
                   ja2=max0(1,i2-ih2)
                   je2=min0(n2,i2+ih2)
                   do 2 j2=ja2,je2
-                     z2=(i2-j2)
+                     z2=(i2-j2)*wght2
                      z2=z2*z2
-                     ih3=dsqrt(hakt2-z1-z2)
+                     ih3=dsqrt(hakt2-z1-z2)/wght3
                      ja3=max0(1,i3-ih3)
                      je3=min0(n3,i3+ih3)
                      do 2 j3=ja3,je3
@@ -585,7 +589,7 @@ C  first stochastic term
                         iz=z
                         az=z-iz
                         wj=kerns(iz+1)*(1-az)+kerns(iz+2)*az
-                        z3=(i3-j3)
+                        z3=(i3-j3)/wght3
                         z3=z3*z3
                         z=1.d2*(z1+z2+z3)/hakt2
                         if(z.ge.1.d2) goto 2
@@ -606,8 +610,8 @@ C   Perform all iterations in local constant bivariate aws for bernoulli variabl
 C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine gbertri(y,n1,n2,n3,hinit,hincr,hmax,lamakt,eta,theta,
-     1               ltheta,lctheta,bi,ai,kernl,kerns,biold,aiold,sym)
-C   
+     1           ltheta,lctheta,bi,ai,kernl,kerns,biold,aiold,sym,wght)
+C
 C   y        observed values of regression function
 C   n1       number of grid-points in first dimension
 C   n2       number of grid-points in second dimension
@@ -615,15 +619,15 @@ C   n3       number of grid-points in second dimension
 C   hinit    initial bandwidth
 C   hincr    factor used to increase bandwiths
 C   hmax     maximal bandwidth
-C   lamakt   lambda*sigma2 
+C   lamakt   lambda*sigma2
 C   eta      memory parameter
 C   theta    estimates            (input)
 C   ltheta   log of estimates    (input)
 C   lctheta  log of 1-estimates    (input)
 C   bi       \sum \Psi^T Wi \Psi  (input/output)
 C   ai       \sum \Psi^T Wi Y     (output)
-C   kernl    discretized localization kernel 
-C   kerns    discretized stochastic kernel 
+C   kernl    discretized localization kernel
+C   kerns    discretized stochastic kernel
 C   biold    working array to store old values of bi
 C   aiold    working array to store old values of ai
 C   sym      asymmetric or symmetric test (logical)
@@ -634,7 +638,7 @@ C
       real*8 y(n1,n2,n3),hinit,hincr,hmax,theta(n1,n2,n3),bi(n1,n2,n3),
      1       ai(n1,n2,n3),kernl(102),kerns(102),lamakt,eta,
      2       biold(n1,n2,n3),aiold(n1,n2,n3),ltheta(n1,n2,n3),
-     3       lctheta(n1,n2,n3),eps
+     3       lctheta(n1,n2,n3),eps,wght(2)
       integer i1,i2,i3
       real*8 hakt,onemeta
       hakt=hinit*hincr
@@ -643,11 +647,11 @@ C
 1     do 21 i1=1,n1
          do 21 i2=1,n2
             do 21 i3=1,n3
-               ltheta(i1,i2,i3)=dlog(theta(i1,i2,i3)+eps)     
-               lctheta(i1,i2,i3)=dlog(1.d0-theta(i1,i2,i3)+eps)     
+               ltheta(i1,i2,i3)=dlog(theta(i1,i2,i3)+eps)
+               lctheta(i1,i2,i3)=dlog(1.d0-theta(i1,i2,i3)+eps)
 21    continue
       call lbertri(y,n1,n2,n3,hakt,lamakt,theta,ltheta,lctheta,
-     1             bi,ai,kernl,kerns,sym)
+     1             bi,ai,kernl,kerns,sym,wght)
       do 31 i1=1,n1
          do 31 i2=1,n2
             do 31 i3=1,n3
@@ -668,12 +672,12 @@ C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine lbermul(n,y,nn,distm,ihakt,theta,ltheta,lctheta,
      1                    bi,bin,ai,ain,lam,h,kernl,kerns,sym)
-C    
+C
 C     n          number of design points
 C     y          observed values at design points
 C     nn         indices of ihinit nearest neighbors
 C     distm      distances ordered by nearest neighbors
-C     ihakt      initial number of nearest neighbors     
+C     ihakt      initial number of nearest neighbors
 C     theta      old estimates from last step (input)
 C     ltheta     log of estimates    (input)
 C     lctheta    log of (1-estimates)    (input)
@@ -683,12 +687,12 @@ C     ai         \sum \Psi^T Wi^(k-1) Y       (input)
 C     ain        \sum \Psi^T Wi^k Y           (output)
 C     lam        lambda*sigma2            (stochastic penalty parameter)
 C     h          actual bandwidth         (location penalty parameter)
-C     kernl      discretized localization kernel 
+C     kernl      discretized localization kernel
 C     kerns      discretized stochastic  (exponential)
 C     sym        asymmetric or symmetric test (logical)
-C     
+C
       implicit logical (a-z)
-      integer n,i,j,iz,ja,o,nwij,ij,ihakt,nn(ihakt,n)      
+      integer n,i,j,iz,ja,ihakt,nn(ihakt,n)
       logical sym
       real*8 y(n),theta(n),kerns(102),bi(n),bin(n),ai(n),ain(n),lam,
      1     kernl(102),distm(ihakt,n),epij,lambda,h,wij,z,az,ha2,bii,
@@ -761,7 +765,7 @@ C     kerns      discretized stochastic (exponential)
 C     sym        asymmetric or symmetric test (logical)
 C     
       implicit logical (a-z)
-      integer n,i,j,iz,ja,o,nwij,ij,ihakt,nn(ihakt,n)      
+      integer n,i,j,iz,ja,ij,ihakt,nn(ihakt,n)      
       logical sym
       real*8 y(n),theta(n),kerns(102),bi(n),bin(n),ai(n),ain(n),lam,
      1    kernl(102),epij,lambda,h,ha,wij,z,az,ha2,bii,
@@ -839,7 +843,7 @@ C
       real*8 y(n),hakt,theta(n),bi(n),ai(n),kernl(102),kerns(102),
      1      lamakt,lam,ltheta(n)
       integer i,j,ja,je,ih,iz
-      real*8 z,z1,wj,az,swj,swjy,bii,thetai,lthetai
+      real*8 z,wj,az,swj,swjy,bii,thetai,lthetai
       ih=hakt
       lam=lamakt*1d-2
       if(sym) lam=2*lam
@@ -879,7 +883,7 @@ C
 C   Perform all iterations in local constant univariate aws for poisson counts 
 C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
-      subroutine gpoiuni(y,n,hinit,hincr,hmax,lamakt,eta,theta,ltheta, 
+      subroutine gpoiuni(y,n,hinit,hincr,hmax,lamakt,eta,theta,ltheta,
      1                   bi,ai,kernl,kerns,biold,aiold,sym)
 C   
 C   y        observed values of regression function
@@ -893,7 +897,7 @@ C   theta    estimates    (output)
 C   bi       \sum \Psi^T Wi \Psi  (output)
 C   ai       \sum \Psi^T Wi Y     (output)
 C   kernl    discretized localization kernel 
-C   kerns    discretized stochastic kernel 
+C   kerns    discretized stochastic kernel
 C   biold    working array to store old values of bi
 C   aiold    working array to store old values of ai
 C   sym      asymmetric or symmetric test (logical)
@@ -930,26 +934,26 @@ C   Perform one iteration in local constant bivariate aws for poisson counts (gr
 C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine lpoibi(y,n1,n2,hakt,lamakt,theta,ltheta,bi,ai,
-     1                  kernl,kerns,sym)
-C   
+     1                  kernl,kerns,sym,wght)
+C
 C   y        observed values of regression function
 C   n1       number of grid-points in first dimension
 C   n2       number of grid-points in second dimension
 C   hakt     actual bandwidth
-C   lamakt   lambda*sigma2 
+C   lamakt   lambda*sigma2
 C   theta    estimates            (input)
 C   ltheta   log of estimates    (input)
 C   bi       \sum \Psi^T Wi \Psi  (input/output)
 C   ai       \sum \Psi^T Wi Y     (output)
-C   kernl    discretized localization kernel 
-C   kerns    discretized stochastic kernel 
+C   kernl    discretized localization kernel
+C   kerns    discretized stochastic kernel
 C   sym      asymmetric or symmetric test (logical)
 C
       implicit logical (a-z)
       integer n1,n2
       logical sym
       real*8 y(n1,n2),hakt,theta(n1,n2),bi(n1,n2),ai(n1,n2),
-     1      kernl(102),kerns(102),lamakt,lam,ltheta(n1,n2)
+     1      kernl(102),kerns(102),lamakt,lam,ltheta(n1,n2),wght
       integer i1,i2,j1,j2,ja1,ja2,je1,je2,ih1,ih2,iz
       real*8 z,z1,z2,wj,az,swj,swjy,bii,thetai,hakt2,lthetai
       ih1=hakt
@@ -968,7 +972,7 @@ C
             do 2 j1=ja1,je1
                z1=(i1-j1)
                z1=z1*z1
-               ih2=dsqrt(hakt2-z1)
+               ih2=dsqrt(hakt2-z1)/wght
                ja2=max0(1,i2-ih2)
                je2=min0(n2,i2+ih2)
                do 2 j2=ja2,je2
@@ -981,7 +985,7 @@ C  first stochastic term
                   iz=z
                   az=z-iz
                   wj=kerns(iz+1)*(1-az)+kerns(iz+2)*az
-                  z2=(i2-j2)
+                  z2=(i2-j2)*wght
                   z=1.d2*(z1+z2*z2)/hakt2
                   if(z.ge.1.d2) goto 2
                   iz=z
@@ -1001,22 +1005,22 @@ C   Perform all iterations in local constant bivariate aws for poisson counts (g
 C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine gpoibi(y,n1,n2,hinit,hincr,hmax,lamakt,eta,theta,
-     1                  ltheta,bi,ai,kernl,kerns,biold,aiold,sym)
-C   
+     1                  ltheta,bi,ai,kernl,kerns,biold,aiold,sym,wght)
+C
 C   y        observed values of regression function
 C   n1       number of grid-points in first dimension
 C   n2       number of grid-points in second dimension
 C   hinit    initial bandwidth
 C   hincr    factor used to increase bandwiths
 C   hmax     maximal bandwidth
-C   lamakt   lambda*sigma2 
+C   lamakt   lambda*sigma2
 C   eta      memory parameter
 C   theta    estimates            (input)
 C   ltheta   log of estimates    (input)
 C   bi       \sum \Psi^T Wi \Psi  (input/output)
 C   ai       \sum \Psi^T Wi Y     (output)
-C   kernl    discretized localization kernel 
-C   kerns    discretized stochastic kernel 
+C   kernl    discretized localization kernel
+C   kerns    discretized stochastic kernel
 C   biold    working array to store old values of bi
 C   aiold    working array to store old values of ai
 C   sym      asymmetric or symmetric test (logical)
@@ -1026,7 +1030,7 @@ C
       logical sym
       real*8 y(n1,n2),hinit,hincr,hmax,theta(n1,n2),bi(n1,n2),
      1       ai(n1,n2),kernl(102),kerns(102),lamakt,eta,biold(n1,n2),
-     2       aiold(n1,n2),ltheta(n1,n2),eps
+     2       aiold(n1,n2),ltheta(n1,n2),eps,wght
       integer i1,i2
       real*8 hakt,onemeta
       hakt=hinit*hincr
@@ -1034,10 +1038,10 @@ C
 1     eps=1.d-50
       do 21 i1=1,n1
          do 21 i2=1,n2
-            ltheta(i1,i2)=dlog(theta(i1,i2)+eps)     
+            ltheta(i1,i2)=dlog(theta(i1,i2)+eps)
 21    continue
       call lpoibi(y,n1,n2,hakt,lamakt,theta,ltheta,bi,ai,
-     1            kernl,kerns,sym)
+     1            kernl,kerns,sym,wght)
       do 31 i1=1,n1
          do 31 i2=1,n2
             ai(i1,i2)=onemeta*ai(i1,i2)+eta*aiold(i1,i2)
@@ -1056,31 +1060,34 @@ C   Perform one iteration in local constant trivariate aws for poisson counts (g
 C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine lpoitri(y,n1,n2,n3,hakt,lamakt,theta,ltheta,bi,ai,
-     1                  kernl,kerns,sym)
-C   
+     1                  kernl,kerns,sym,wght)
+C
 C   y        observed values of regression function
 C   n1       number of grid-points in first dimension
 C   n2       number of grid-points in second dimension
 C   n3       number of grid-points in third dimension
 C   hakt     actual bandwidth
-C   lamakt   lambda*sigma2 
+C   lamakt   lambda*sigma2
 C   theta    estimates            (input)
 C   ltheta   log of estimates    (input)
 C   bi       \sum \Psi^T Wi \Psi  (input/output)
 C   ai       \sum \Psi^T Wi Y     (output)
-C   kernl    discretized localization kernel 
-C   kerns    discretized stochastic kernel 
+C   kernl    discretized localization kernel
+C   kerns    discretized stochastic kernel
 C   sym      asymmetric or symmetric test (logical)
 C
       implicit logical (a-z)
       integer n1,n2,n3
       logical sym
       real*8 y(n1,n2,n3),hakt,theta(n1,n2,n3),bi(n1,n2,n3),kernl(102),
-     1      ai(n1,n2,n3),kerns(102),lamakt,lam,ltheta(n1,n2,n3)
+     1      ai(n1,n2,n3),kerns(102),lamakt,lam,ltheta(n1,n2,n3),wght(2)
       integer i1,i2,i3,j1,j2,j3,ja1,ja2,ja3,je1,je2,je3,ih1,ih2,ih3,iz
-      real*8 z,z1,z2,z3,wj,az,swj,swjy,bii,thetai,hakt2,lthetai
+      real*8 z,z1,z2,z3,wj,az,swj,swjy,bii,thetai,hakt2,lthetai,wght2,
+     1       wght3
       ih1=hakt
       hakt2=hakt*hakt
+      wght2=wght(1)
+      wght3=wght(2)
       lam=lamakt*1d-2
       if(sym) lam=2*lam
       do 1 i1=1,n1
@@ -1096,13 +1103,13 @@ C
                do 2 j1=ja1,je1
                   z1=(i1-j1)
                   z1=z1*z1
-                  ih2=dsqrt(hakt2-z1)
+                  ih2=dsqrt(hakt2-z1)/wght2
                   ja2=max0(1,i2-ih2)
                   je2=min0(n2,i2+ih2)
                   do 2 j2=ja2,je2
-                     z2=(i2-j2)
+                     z2=(i2-j2)*wght2
                      z2=z2*z2
-                     ih3=dsqrt(hakt2-z1-z2)
+                     ih3=dsqrt(hakt2-z1-z2)/wght3
                      ja3=max0(1,i3-ih3)
                      je3=min0(n3,i3+ih3)
                      do 2 j3=ja3,je3
@@ -1116,7 +1123,7 @@ C  first stochastic term
                         iz=z
                         az=z-iz
                         wj=kerns(iz+1)*(1-az)+kerns(iz+2)*az
-                        z3=(i3-j3)
+                        z3=(i3-j3)*wght3
                         z3=z3*z3
                         z=1.d2*(z1+z2+z3)/hakt2
                         if(z.ge.1.d2) goto 2
@@ -1137,8 +1144,8 @@ C   Perform all iterations in local constant bivariate aws for poisson counts (g
 C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine gpoitri(y,n1,n2,n3,hinit,hincr,hmax,lamakt,eta,theta,
-     1                   ltheta,bi,ai,kernl,kerns,biold,aiold,sym)
-C   
+     1                   ltheta,bi,ai,kernl,kerns,biold,aiold,sym,wght)
+C
 C   y        observed values of regression function
 C   n1       number of grid-points in first dimension
 C   n2       number of grid-points in second dimension
@@ -1146,14 +1153,14 @@ C   n3       number of grid-points in second dimension
 C   hinit    initial bandwidth
 C   hincr    factor used to increase bandwiths
 C   hmax     maximal bandwidth
-C   lamakt   lambda*sigma2 
+C   lamakt   lambda*sigma2
 C   eta      memory parameter
 C   theta    estimates            (input)
 C   ltheta   log of estimates    (input)
 C   bi       \sum \Psi^T Wi \Psi  (input/output)
 C   ai       \sum \Psi^T Wi Y     (output)
-C   kernl    discretized localization kernel 
-C   kerns    discretized stochastic kernel 
+C   kernl    discretized localization kernel
+C   kerns    discretized stochastic kernel
 C   biold    working array to store old values of bi
 C   aiold    working array to store old values of ai
 C   sym      asymmetric or symmetric test (logical)
@@ -1162,7 +1169,7 @@ C
       integer n1,n2,n3
       logical sym
       real*8 y(n1,n2,n3),hinit,hincr,hmax,theta(n1,n2,n3),bi(n1,n2,n3),
-     1       ai(n1,n2,n3),kernl(102),kerns(102),lamakt,eta,
+     1       ai(n1,n2,n3),kernl(102),kerns(102),lamakt,eta,wght(2),
      2       biold(n1,n2,n3),aiold(n1,n2,n3),ltheta(n1,n2,n3),eps
       integer i1,i2,i3
       real*8 hakt,onemeta
@@ -1172,10 +1179,10 @@ C
       do 21 i1=1,n1
          do 21 i2=1,n2
             do 21 i3=1,n3
-               ltheta(i1,i2,i3)=dlog(theta(i1,i2,i3)+eps)     
+               ltheta(i1,i2,i3)=dlog(theta(i1,i2,i3)+eps)
 21    continue
       call lpoitri(y,n1,n2,n3,hakt,lamakt,theta,ltheta,bi,ai,kernl,
-     1             kerns,sym)
+     1             kerns,sym,wght)
       do 31 i1=1,n1
          do 31 i2=1,n2
             do 31 i3=1,n3
@@ -1196,12 +1203,12 @@ C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine lpoimul(n,y,nn,distm,ihakt,theta,ltheta,
      1                    bi,bin,ai,ain,lam,h,kernl,kerns,sym)
-C    
+C
 C     n          number of design points
 C     y          observed values at design points
 C     nn         indices of ihinit nearest neighbors
 C     distm      distances ordered by nearest neighbors
-C     ihakt      initial number of nearest neighbors     
+C     ihakt      initial number of nearest neighbors
 C     theta      old estimates from last step (input)
 C     ltheta   log of estimates    (input)
 C     bi         \sum \Psi^T Wi^(k-1) \Psi    (input)
@@ -1210,18 +1217,18 @@ C     ai         \sum \Psi^T Wi^(k-1) Y       (input)
 C     ain        \sum \Psi^T Wi^k Y           (output)
 C     lam        lambda*sigma2            (stochastic penalty parameter)
 C     h          actual bandwidth         (location penalty parameter)
-C     kernl      discretized localization kernel 
+C     kernl      discretized localization kernel
 C     kerns      discretized stochastic  (exponential)
 C     sym        asymmetric or symmetric test (logical)
-C     
+C
       implicit logical (a-z)
-      integer n,i,j,iz,ja,o,nwij,ij,ihakt,nn(ihakt,n)      
+      integer n,i,j,iz,ja,ihakt,nn(ihakt,n)
       logical sym
       real*8 y(n),theta(n),kerns(102),bi(n),bin(n),ai(n),ain(n),lam,
      1     kernl(102),distm(ihakt,n),epij,lambda,h,wij,z,az,ha2,bii,
      2     ltheta(n),thetai,lthetai
 
-C     
+C
 C     in case of dp1==1  lawsuni should be preferred  (p=0)
 C
       ha2=h*h/1.d2
@@ -1267,7 +1274,7 @@ C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine lpoimnn(n,y,nn,ihakt,theta,ltheta,bi,bin,
      1                    ai,ain,lam,h,kernl,kerns,sym)
-C    
+C
 C     n          number of design points
 C     y          observed values at design points
 C     nn         indices of ihinit nearest neighbors
@@ -1285,7 +1292,7 @@ C     kerns      discretized stochastic  (exponential)
 C     sym        asymmetric or symmetric test (logical)
 C     
       implicit logical (a-z)
-      integer n,i,j,iz,ja,o,nwij,ij,ihakt,nn(ihakt,n)   
+      integer n,i,j,iz,ja,ij,ihakt,nn(ihakt,n)   
       logical sym   
       real*8 y(n),theta(n),kerns(102),bi(n),bin(n),ai(n),ain(n),lam,
      1    kernl(102),epij,lambda,h,ha,wij,z,az,ha2,bii,
@@ -1360,7 +1367,7 @@ C
       real*8 y(n),hakt,theta(n),bi(n),ai(n),kernl(102),kerns(102),
      1      lamakt,lam,ltheta(n)
       integer i,j,ja,je,ih,iz
-      real*8 z,z1,wj,az,swj,swjy,bii,thetai,lthetai,hakt0
+      real*8 z,wj,az,swj,swjy,bii,thetai,lthetai,hakt0
       ih=hakt
       hakt0=hakt*1.d-1
       lam=lamakt*1.d-2
@@ -1401,7 +1408,7 @@ C
 C   Perform all iterations in local constant univariate aws for exponential variates 
 C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
-      subroutine gexpuni(y,n,hinit,hincr,hmax,lamakt,eta,theta,ltheta, 
+      subroutine gexpuni(y,n,hinit,hincr,hmax,lamakt,eta,theta,ltheta,
      1                   bi,ai,kernl,kerns,biold,aiold,sym)
 C   
 C   y        observed values of regression function
@@ -1452,26 +1459,26 @@ C   Perform one iteration in local constant bivariate aws for exponential variat
 C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine lexpbi(y,n1,n2,hakt,lamakt,theta,ltheta,bi,ai,
-     1                  kernl,kerns,sym)
-C   
+     1                  kernl,kerns,sym,wght)
+C
 C   y        observed values of regression function
 C   n1       number of grid-points in first dimension
 C   n2       number of grid-points in second dimension
 C   hakt     actual bandwidth
-C   lamakt   lambda*sigma2 
+C   lamakt   lambda*sigma2
 C   theta    estimates            (input)
 C   ltheta   log of estimates    (input)
 C   bi       \sum \Psi^T Wi \Psi  (input/output)
 C   ai       \sum \Psi^T Wi Y     (output)
-C   kernl    discretized localization kernel 
-C   kerns    discretized stochastic kernel 
+C   kernl    discretized localization kernel
+C   kerns    discretized stochastic kernel
 C   sym      asymmetric or symmetric test (logical)
 C
       implicit logical (a-z)
       integer n1,n2
       logical sym
       real*8 y(n1,n2),hakt,theta(n1,n2),bi(n1,n2),ai(n1,n2),
-     1      kernl(102),kerns(102),lamakt,lam,ltheta(n1,n2)
+     1      kernl(102),kerns(102),lamakt,lam,ltheta(n1,n2),wght
       integer i1,i2,j1,j2,ja1,ja2,je1,je2,ih1,ih2,iz
       real*8 z,z1,z2,wj,az,swj,swjy,bii,thetai,hakt2,lthetai
       ih1=hakt
@@ -1490,7 +1497,7 @@ C
             do 2 j1=ja1,je1
                z1=(i1-j1)
                z1=z1*z1
-               ih2=dsqrt(hakt2-z1)
+               ih2=dsqrt(hakt2-z1)/wght
                ja2=max0(1,i2-ih2)
                je2=min0(n2,i2+ih2)
                do 2 j2=ja2,je2
@@ -1503,7 +1510,7 @@ C  first stochastic term
                   iz=z
                   az=z-iz
                   wj=kerns(iz+1)*(1-az)+kerns(iz+2)*az
-                  z2=(i2-j2)
+                  z2=(i2-j2)*wght
                   z=1.d2*(z1+z2*z2)/hakt2
                   if(z.ge.1.d2) goto 2
                   iz=z
@@ -1523,22 +1530,22 @@ C   Perform all iterations in local constant bivariate aws for exponential varia
 C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine gexpbi(y,n1,n2,hinit,hincr,hmax,lamakt,eta,theta,
-     1                  ltheta,bi,ai,kernl,kerns,biold,aiold,sym)
-C   
+     1                  ltheta,bi,ai,kernl,kerns,biold,aiold,sym,wght)
+C
 C   y        observed values of regression function
 C   n1       number of grid-points in first dimension
 C   n2       number of grid-points in second dimension
 C   hinit    initial bandwidth
 C   hincr    factor used to increase bandwiths
 C   hmax     maximal bandwidth
-C   lamakt   lambda*sigma2 
+C   lamakt   lambda*sigma2
 C   eta      memory parameter
 C   theta    estimates            (input)
 C   ltheta   log of estimates    (input)
 C   bi       \sum \Psi^T Wi \Psi  (input/output)
 C   ai       \sum \Psi^T Wi Y     (output)
-C   kernl    discretized localization kernel 
-C   kerns    discretized stochastic kernel 
+C   kernl    discretized localization kernel
+C   kerns    discretized stochastic kernel
 C   biold    working array to store old values of bi
 C   aiold    working array to store old values of ai
 C   sym      asymmetric or symmetric test (logical)
@@ -1548,7 +1555,7 @@ C
       logical sym
       real*8 y(n1,n2),hinit,hincr,hmax,theta(n1,n2),bi(n1,n2),
      1       ai(n1,n2),kernl(102),kerns(102),lamakt,eta,biold(n1,n2),
-     2       aiold(n1,n2),ltheta(n1,n2),eps
+     2       aiold(n1,n2),ltheta(n1,n2),eps,wght
       integer i1,i2
       real*8 hakt,onemeta
       hakt=hinit*hincr
@@ -1556,10 +1563,10 @@ C
 1     eps=1.d-50
       do 21 i1=1,n1
          do 21 i2=1,n2
-            ltheta(i1,i2)=dlog(theta(i1,i2)+eps)     
+            ltheta(i1,i2)=dlog(theta(i1,i2)+eps)
 21    continue
       call lexpbi(y,n1,n2,hakt,lamakt,theta,ltheta,bi,ai,kernl,
-     1            kerns,sym)
+     1            kerns,sym,wght)
       do 31 i1=1,n1
          do 31 i2=1,n2
             ai(i1,i2)=onemeta*ai(i1,i2)+eta*aiold(i1,i2)
@@ -1578,31 +1585,34 @@ C   Perform one iteration in local constant trivariate aws for exponential varia
 C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine lexptri(y,n1,n2,n3,hakt,lamakt,theta,ltheta,bi,ai,
-     1                  kernl,kerns,sym)
-C   
+     1                  kernl,kerns,sym,wght)
+C
 C   y        observed values of regression function
 C   n1       number of grid-points in first dimension
 C   n2       number of grid-points in second dimension
 C   n3       number of grid-points in third dimension
 C   hakt     actual bandwidth
-C   lamakt   lambda*sigma2 
+C   lamakt   lambda*sigma2
 C   theta    estimates            (input)
 C   ltheta   log of estimates    (input)
 C   bi       \sum \Psi^T Wi \Psi  (input/output)
 C   ai       \sum \Psi^T Wi Y     (output)
-C   kernl    discretized localization kernel 
-C   kerns    discretized stochastic kernel 
+C   kernl    discretized localization kernel
+C   kerns    discretized stochastic kernel
 C   sym      asymmetric or symmetric test (logical)
 C
       implicit logical (a-z)
       integer n1,n2,n3
       logical sym
       real*8 y(n1,n2,n3),hakt,theta(n1,n2,n3),bi(n1,n2,n3),kernl(102),
-     1      ai(n1,n2,n3),kerns(102),lamakt,lam,ltheta(n1,n2,n3)
+     1      ai(n1,n2,n3),kerns(102),lamakt,lam,ltheta(n1,n2,n3),wght(2)
       integer i1,i2,i3,j1,j2,j3,ja1,ja2,ja3,je1,je2,je3,ih1,ih2,ih3,iz
-      real*8 z,z1,z2,z3,wj,az,swj,swjy,bii,thetai,hakt2,lthetai
+      real*8 z,z1,z2,z3,wj,az,swj,swjy,bii,thetai,hakt2,lthetai,wght2,
+     1       wght3
       ih1=hakt
       hakt2=hakt*hakt
+      wght2=wght(1)
+      wght3=wght(2)
       lam=lamakt*1d-2
       if(sym) lam=2*lam
       do 1 i1=1,n1
@@ -1618,13 +1628,13 @@ C
                do 2 j1=ja1,je1
                   z1=(i1-j1)
                   z1=z1*z1
-                  ih2=dsqrt(hakt2-z1)
+                  ih2=dsqrt(hakt2-z1)/wght2
                   ja2=max0(1,i2-ih2)
                   je2=min0(n2,i2+ih2)
                   do 2 j2=ja2,je2
-                     z2=(i2-j2)
+                     z2=(i2-j2)*wght2
                      z2=z2*z2
-                     ih3=dsqrt(hakt2-z1-z2)
+                     ih3=dsqrt(hakt2-z1-z2)/wght3
                      ja3=max0(1,i3-ih3)
                      je3=min0(n3,i3+ih3)
                      do 2 j3=ja3,je3
@@ -1637,7 +1647,7 @@ C  first stochastic term
                         iz=z
                         az=z-iz
                         wj=kerns(iz+1)*(1-az)+kerns(iz+2)*az
-                        z3=(i3-j3)
+                        z3=(i3-j3)*wght3
                         z3=z3*z3
                         z=1.d2*(z1+z2+z3)/hakt2
                         if(z.ge.1.d2) goto 2
@@ -1658,8 +1668,8 @@ C   Perform all iterations in local constant bivariate aws for exponential varia
 C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine gexptri(y,n1,n2,n3,hinit,hincr,hmax,lamakt,eta,theta,
-     1                   ltheta,bi,ai,kernl,kerns,biold,aiold,sym)
-C   
+     1                   ltheta,bi,ai,kernl,kerns,biold,aiold,sym,wght)
+C
 C   y        observed values of regression function
 C   n1       number of grid-points in first dimension
 C   n2       number of grid-points in second dimension
@@ -1667,14 +1677,14 @@ C   n3       number of grid-points in second dimension
 C   hinit    initial bandwidth
 C   hincr    factor used to increase bandwiths
 C   hmax     maximal bandwidth
-C   lamakt   lambda*sigma2 
+C   lamakt   lambda*sigma2
 C   eta      memory parameter
 C   theta    estimates            (input)
 C   ltheta   log of estimates    (input)
 C   bi       \sum \Psi^T Wi \Psi  (input/output)
 C   ai       \sum \Psi^T Wi Y     (output)
-C   kernl    discretized localization kernel 
-C   kerns    discretized stochastic kernel 
+C   kernl    discretized localization kernel
+C   kerns    discretized stochastic kernel
 C   biold    working array to store old values of bi
 C   aiold    working array to store old values of ai
 C   sym      asymmetric or symmetric test (logical)
@@ -1683,7 +1693,7 @@ C
       integer n1,n2,n3
       logical sym
       real*8 y(n1,n2,n3),hinit,hincr,hmax,theta(n1,n2,n3),bi(n1,n2,n3),
-     1       ai(n1,n2,n3),kernl(102),kerns(102),lamakt,eta,
+     1       ai(n1,n2,n3),kernl(102),kerns(102),lamakt,eta,wght(2),
      2       biold(n1,n2,n3),aiold(n1,n2,n3),ltheta(n1,n2,n3),eps
       integer i1,i2,i3
       real*8 hakt,onemeta
@@ -1693,10 +1703,10 @@ C
       do 21 i1=1,n1
          do 21 i2=1,n2
             do 21 i3=1,n3
-               ltheta(i1,i2,i3)=dlog(theta(i1,i2,i3)+eps)     
+               ltheta(i1,i2,i3)=dlog(theta(i1,i2,i3)+eps)
 21    continue
       call lexptri(y,n1,n2,n3,hakt,lamakt,theta,ltheta,bi,ai,kernl,
-     1             kerns,sym)
+     1             kerns,sym,wght)
       do 31 i1=1,n1
          do 31 i2=1,n2
             do 31 i3=1,n3
@@ -1717,12 +1727,12 @@ C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine lexpmul(n,y,nn,distm,ihakt,theta,ltheta,
      1                    bi,bin,ai,ain,lam,h,kernl,kerns,sym)
-C    
+C
 C     n          number of design points
 C     y          observed values at design points
 C     nn         indices of ihinit nearest neighbors
 C     distm      distances ordered by nearest neighbors
-C     ihakt      initial number of nearest neighbors     
+C     ihakt      initial number of nearest neighbors
 C     theta      old estimates from last step (input)
 C     ltheta     log of estimates    (input)
 C     bi         \sum \Psi^T Wi^(k-1) \Psi    (input)
@@ -1731,17 +1741,17 @@ C     ai         \sum \Psi^T Wi^(k-1) Y       (input)
 C     ain        \sum \Psi^T Wi^k Y           (output)
 C     lam        lambda*sigma2            (stochastic penalty parameter)
 C     h          actual bandwidth         (location penalty parameter)
-C     kernl      discretized localization kernel 
+C     kernl      discretized localization kernel
 C     kerns      discretized stochastic  (exponential)
 C     sym        asymmetric or symmetric test (logical)
-C     
+C
       implicit logical (a-z)
-      integer n,i,j,iz,ja,o,nwij,ij,ihakt,nn(ihakt,n) 
-      logical sym     
+      integer n,i,j,iz,ja,ihakt,nn(ihakt,n)
+      logical sym
       real*8 y(n),theta(n),kerns(102),bi(n),bin(n),ai(n),ain(n),lam,
      1     kernl(102),distm(ihakt,n),epij,lambda,h,wij,z,az,ha2,bii,
      2     ltheta(n),thetai,lthetai
-C     
+C
 C     in case of dp1==1  lawsuni should be preferred  (p=0)
 C
       ha2=h*h/1.d2
@@ -1804,7 +1814,7 @@ C     kerns      discretized stochastic  (exponential)
 C     sym        asymmetric or symmetric test (logical)
 C     
       implicit logical (a-z)
-      integer n,i,j,iz,ja,o,nwij,ij,ihakt,nn(ihakt,n)      
+      integer n,i,j,iz,ja,ij,ihakt,nn(ihakt,n)      
       logical sym
       real*8 y(n),theta(n),kerns(102),bi(n),bin(n),ai(n),ain(n),lam,
      1    kernl(102),epij,lambda,h,ha,wij,z,az,ha2,bii,
